#!/usr/bin/python

import numpy as np
import os
from pylab import *
from sympy import var
from scipy.optimize import newton
from sys import argv
from exampleproblems import *

from regularizationTools import *

f = open('errorTable.tex', 'w')

def bError(b, normNoise):
    """
    Returns an error contaminated b vector based on the norm of the noise.
    """
    n, = np.shape(b)
    # Create an error of norm normNoise
    err = np.random.randn(n)
    err = err / np.linalg.norm(err,2)
    err = normNoise * np.linalg.norm(b) * err
    #print b
    #print b+err
    return b + err

def makeGraphs(x, xMu, xMuTik, xTSVD, normNoise, graphTitle, testName):
    global f
    figure()
    n, = np.shape(x)
    plot (arange(0,n), x,      '-',   label="Exact Solution")
    plot (arange(0,n), xMu,    'r-',  label="Proposed Method")
    plot (arange(0,n), xMuTik, 'g--', label="Tikhonov")
    plot (arange(0,n), xTSVD,  'b-.',  label="TSVD")
    titleString = graphTitle 
    #legend()
    #title(titleString)
    savefig("graphs/" + testName + "_" + str(int(normNoise*100)) + ".png")

def runTests(test, numTests, n, graphs=False):
    global f
    tests = xrange(0,numTests)
    normNoiseList = [.1,.01,.001]
    A,x,b = test(n)
    b = np.dot(A,x)
    f.write( "\\subsection*{"+test.__name__+"}" + "\n")
    f.write( "\\begin{tabular}{lllllllll}"  + "\n")
    f.write(  "\\bf{$Percent Error$} & \\bf{Lothar} & \\bf{LotharRG} & \\bf{LotharAlternate} & \\bf{Tikhonov} & \\bf{TikhonovRG} & \\bf{TSVD} & \\bf{TSVDAlternate} \\\\" + "\n")
    f.write(  "\\hline\\\\" + "\n")
    print "Begining test", test.__name__
    for percentNoise in normNoiseList:
        xMuTik   = np.zeros(n)
        xMuTikRG = np.zeros(n)
        xMu      = np.zeros(n)
        xMuRG    = np.zeros(n)
        xMuAlt   = np.zeros(n)
        xTSVD    = np.zeros(n)
        xTSVDTik = np.zeros(n)
        xTSVDAlt = np.zeros(n)
        for t in tests:
            print "Testing", t+1, "of", len(tests), "in", percentNoise
            berr = bError(b, percentNoise)                          # error contaminated vector
            normNoise = percentNoise * np.linalg.norm(b)
            xMuTik   += solveTikhonov       (A, berr, normNoise) # standard tikhonov solution
            xMuTikRG += solveTikhonovRG     (A, berr, normNoise) # tikhonov solution using RG method
            xMu      += solveLothar         (A, berr, normNoise) # lothar's solution
            xMuRG    += solveLotharRG       (A, berr, normNoise) # lothar's solution using RG method
            xMuAlt   += solveLotharAlternate(A, berr, normNoise) # lothar's solution by truncating singular values after mu
            xTSVD    += solveTSVD           (A, berr, normNoise) # TSVD solution
            xTSVDTik += solveTSVDTikhonov   (A, berr, normNoise) # truncated TSVD solution by setting sigma less than mu to zero
            xTSVDAlt += solveTSVDAlternate  (A, berr, normNoise) # truncated TSVD solution by setting sigma less than mu to zero
        
        xMuTik   = xMuTik   / len(tests)
        xMuTikRG = xMuTikRG / len(tests)
        xMu      = xMu      / len(tests)
        xMuRG    = xMuRG    / len(tests)
        xMuAlt   = xMuAlt   / len(tests)
        xTSVD    = xTSVD    / len(tests)
        xTSVDTik = xTSVDTik / len(tests)
        xTSVDAlt = xTSVDAlt / len(tests)

        lotharResidual    = np.linalg.norm(x - xMu,      2) / np.linalg.norm(x)
        lotharRGResidual  = np.linalg.norm(x - xMuRG,    2) / np.linalg.norm(x)
        lotharAltResidual = np.linalg.norm(x - xMuAlt,   2) / np.linalg.norm(x)
        tikResidual       = np.linalg.norm(x - xMuTik,   2) / np.linalg.norm(x)
        tikRGResidual     = np.linalg.norm(x - xMuTikRG, 2) / np.linalg.norm(x)
        TSVDResidual      = np.linalg.norm(x - xTSVD,    2) / np.linalg.norm(x)
        TSVDTikResidual   = np.linalg.norm(x - xTSVDTik, 2) / np.linalg.norm(x)
        TSVDAltResidual   = np.linalg.norm(x - xTSVDAlt, 2) / np.linalg.norm(x)
    
        f.write(  str(percentNoise * 100) + "\\%" + "&"+ str(lotharResidual) + "&" + str(lotharRGResidual) + "&" + str(lotharAltResidual) + "&"+ str(tikResidual) + "&" + str(tikRGResidual) + "&"+ str(TSVDResidual) + "&"+ str(TSVDAltResidual) + "\\\\"+ "\n")
        if graphs:
            makeGraphs(x, xMu, xMuTik, xTSVD, xTSVDAlt, percentNoise, test.__name__)
            #bx      = np.dot(A,x)
            #bxMu    = np.dot(A,xMu)
            #bxMuTik = np.dot(A,xMuTik)
            #bxTSVD  = np.dot(A,xTSVD)
            #bxTSVDTik = np.dot(A,xTSVDTik)
    #        makeGraphs(bx, bxMu, bxMuTik, bxTSVD, bxTSVDTik, normNoise, test.__name__ + "Solution")
    f.write(  "\\end{tabular}" + "\n")

def runOptimalTests(test, name, numTests, n, graphs=False):
    global f
    tests = xrange(0,numTests)
    normNoiseList = [.1,.05,.01,.001]
    A,x,b = test(n)
    b = np.dot(A,x)
    f.write( "\\subsection*{"+test.__name__+"}" + "\n")
    f.write( "\\begin{tabular}{llll}"  + "\n")
    f.write(  "\\bf{Percent Error} & \\bf{Lothar} & \\bf{Tikhonov} & \\bf{TSVD} \\\\" + "\n")
    f.write(  "\\hline\\\\" + "\n")
    print "Begining test", test.__name__
    for percentNoise in normNoiseList:
        xMuTik   = np.zeros(n)
        xMu      = np.zeros(n)
        xTSVD    = np.zeros(n)
        for t in tests:
            print "Testing", t+1, "of", len(tests), "in", percentNoise
            berr = bError(b, percentNoise)                          # error contaminated vector
            normNoise = percentNoise * np.linalg.norm(b)
            xMuTik   += solveTikhonovOptimal(A, berr, x, normNoise) # standard tikhonov solution
            xMu      += solveLotharOptimal  (A, berr, x, normNoise) # lothar's solution
            xTSVD    += solveTSVD           (A, berr, normNoise) # TSVD solution
        
        xMuTik   = xMuTik   / len(tests)
        xMu      = xMu      / len(tests)
        xTSVD    = xTSVD    / len(tests)

        lotharResidual    = np.linalg.norm(x - xMu,      2) / np.linalg.norm(x)
        tikResidual       = np.linalg.norm(x - xMuTik,   2) / np.linalg.norm(x)
        TSVDResidual      = np.linalg.norm(x - xTSVD,    2) / np.linalg.norm(x)
    
        f.write(  str(percentNoise * 100) + "\\%" + "&"+ str(lotharResidual) + "&"+ str(tikResidual) + "&"+ str(TSVDResidual) +  "\\\\"+ "\n")
        if graphs:
            makeGraphs(x, xMu, xMuTik, xTSVD, percentNoise, name + " Test Problem with " + str(percentNoise * 100) + "% Noise", test.__name__)
    f.write(  "\\end{tabular}" + "\n")

def runFinalTests(test, name, numTests, n, graphs=False):
    global f
    tests = xrange(0,numTests)
    normNoiseList = [.1,.05,.01,.001]
    A,x,b = test(n)
    b = np.dot(A,x)
    f.write( "\\subsection*{"+test.__name__+"}" + "\n")
    f.write( "\\begin{tabular}{llll}"  + "\n")
    f.write(  "\\bf{Percent Error} & \\bf{Lothar} & \\bf{Tikhonov} & \\bf{TSVD} \\\\" + "\n")
    f.write(  "\\hline\\\\" + "\n")
    print "Begining test", test.__name__
    for percentNoise in normNoiseList:
        xMuTik   = np.zeros(n)
        xMu      = np.zeros(n)
        xTSVD    = np.zeros(n)
        for t in tests:
            print "Testing", t+1, "of", len(tests), "in", percentNoise
            berr = bError(b, percentNoise)                          # error contaminated vector
            normNoise = percentNoise * np.linalg.norm(b)
            xMuTik   += solveTikhonov       (A, berr, normNoise) # standard tikhonov solution
            xMu      += solveLothar         (A, berr, normNoise) # lothar's solution
            xTSVD    += solveTSVD           (A, berr, normNoise) # TSVD solution
        
        xMuTik   = xMuTik   / len(tests)
        xMu      = xMu      / len(tests)
        xTSVD    = xTSVD    / len(tests)

        lotharResidual    = np.linalg.norm(x - xMu,      2) / np.linalg.norm(x)
        tikResidual       = np.linalg.norm(x - xMuTik,   2) / np.linalg.norm(x)
        TSVDResidual      = np.linalg.norm(x - xTSVD,    2) / np.linalg.norm(x)
    
        f.write(  str(percentNoise * 100) + "\\%" + "&"+ str(lotharResidual) + "&"+ str(tikResidual) + "&"+ str(TSVDResidual) +  "\\\\"+ "\n")
        if graphs:
            makeGraphs(x, xMu, xMuTik, xTSVD, percentNoise, name + " Test Problem with " + str(percentNoise * 100) + "% Noise", test.__name__)
    f.write(  "\\end{tabular}" + "\n")

if __name__=="__main__":
    n = 200
    numTests = 1000
    f.write( '\\documentclass[11pt]{article}' + "\n")
    f.write( '\\usepackage[hmargin=1cm,vmargin=1.5cm,landscape]{geometry}' + "\n")
    f.write( '\\begin{document}' + "\n")
    f.write( 'The following results are averaged over ' + str(numTests) + ' tests discretized to $' + str(n) + ' \\times ' + str(n) + '$ matrices.\n\n'  + \
             'The method Lothar refers to replacing $\sigma_j^2$ with $\mu$ when $\sigma_j^2 < \mu$.\n\n' + \
             'The method LotharRG does the same thing, except that $\mu$ is determined by the RG method.\n\n' + \
             'The method LotharAlternate uses $\mu$ from Tikhonov (not RG) and replaces $\sigma_j^2$ with $\mu$ if $\sigma_j^2$ is closer to $\mu$ than it is to 0. Otherwise, it replaces $\sigma_j^2$ with 0.\n\n' + \
             'The method Tikhonov is standard Tikhonov regularization.\n\n' + \
             'The method TikhonovRG is standard Tikhonov regularization with $\mu$ determined by the RG method.\n\n' + \
             'The method TSVDTikhonov replaces $\sigma_j^2$ with 0 if $\sigma_j^2 < \mu$.\n\n' + \
             'The method TSVDAtlernate truncates at some $\sigma_j$ and replaces the rest of the singular values with $\sigma_j$\n\n.')
    g = False
    if len(argv) > 1:
        if str(argv[1]) == "g":
            g = True
    testList = [phillips, shaw, ilaplace, deriv2, baart]
    nameList =  ["Phillips", "Shaw", "Inverse Laplace", "Fredholm Integral", "Baart"]
    for test,name in zip(testList,nameList):
        runFinalTests(test,name, numTests, n, graphs=g)
    if g:
        show()
    f.write( '\\end{document}' + "\n")
    f.close()
    os.system('pdflatex errorTable.tex')
    os.system('evince errorTable.pdf &')
